// Copyright 2009 the V8 project authors. All rights reserved.
//  @2013 Modifications to fit minimal needs for S8 project by
//	Alejandro Fabian Reimondo - aleReimondo@smalltalking.net
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are
// met:
//
//     * Redistributions of source code must retain the above copyright
//       notice, this list of conditions and the following disclaimer.
//     * Redistributions in binary form must reproduce the above
//       copyright notice, this list of conditions and the following
//       disclaimer in the documentation and/or other materials provided
//       with the distribution.
//     * Neither the name of Google Inc. nor the names of its
//       contributors may be used to endorse or promote products derived
//       from this software without specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
// LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
// A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
// OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
// SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
// LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
// DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
// THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
// OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

// This code is based off of the shell.cc sample application in the v8 source tree.
// It has been modified by Alejandro Fabian Reimondo
//  to implement the default shell for console based S8 ambiences.

#include <include/v8.h>

#include <include/libplatform/libplatform.h>

#include <assert.h>
#include <fcntl.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

//Ale for file dateTime
#include <sys/stat.h>
#include <time.h>

#ifdef COMPRESS_STARTUP_DATA_BZ2
#error Using compressed startup data is not supported for this sample
#endif

v8::Handle<v8::Context> CreateShellContext(v8::Isolate* isolate);
void RunShell(v8::Handle<v8::Context> context);
int RunMain(v8::Isolate* isolate, int argc, char* argv[]);
bool ExecuteString(v8::Isolate* isolate,
                   v8::Handle<v8::String> source,
                   v8::Handle<v8::Value> name,
                   bool print_result,
                   bool report_exceptions);
void Print(const v8::FunctionCallbackInfo<v8::Value>& args);
void Read(const v8::FunctionCallbackInfo<v8::Value>& args);
void Load(const v8::FunctionCallbackInfo<v8::Value>& args);
void Quit(const v8::FunctionCallbackInfo<v8::Value>& args);
void Version(const v8::FunctionCallbackInfo<v8::Value>& args);

void Int8Array(const v8::FunctionCallbackInfo<v8::Value>& args);
void Uint8Array(const v8::FunctionCallbackInfo<v8::Value>& args);
void Int16Array(const v8::FunctionCallbackInfo<v8::Value>& args);
void Uint16Array(const v8::FunctionCallbackInfo<v8::Value>& args);
void Int32Array(const v8::FunctionCallbackInfo<v8::Value>& args);
void Uint32Array(const v8::FunctionCallbackInfo<v8::Value>& args);
void Float32Array(const v8::FunctionCallbackInfo<v8::Value>& args);
void Float64Array(const v8::FunctionCallbackInfo<v8::Value>& args);
void PixelArray(const v8::FunctionCallbackInfo<v8::Value>& args);

void FileContents(const v8::FunctionCallbackInfo<v8::Value>& args);
void FileWrite(const v8::FunctionCallbackInfo<v8::Value>& args);
void DebugBreak(const v8::FunctionCallbackInfo<v8::Value>& args);
void IsFileOlderThan(const v8::FunctionCallbackInfo<v8::Value>& args);

v8::Handle<v8::String> ReadFile(v8::Isolate* isolate, const char* name);
void ReportException(v8::Isolate* isolate, v8::TryCatch* handler);

static bool run_shell;

class S8ArrayBufferAllocator : public v8::ArrayBuffer::Allocator {
 public:
  virtual void* Allocate(size_t length) {
    void* data = AllocateUninitialized(length);
    return data == NULL ? data : memset(data, 0, length);
  }
  virtual void* AllocateUninitialized(size_t length) { return malloc(length); }
  virtual void Free(void* data, size_t) { free(data); }
};


int main(int argc, char* argv[]) {
  v8::V8::InitializeICU();
  v8::Platform* platform = v8::platform::CreateDefaultPlatform();
  v8::V8::InitializePlatform(platform);
  v8::V8::Initialize();
  v8::V8::SetFlagsFromCommandLine(&argc, argv, true);
  S8ArrayBufferAllocator array_buffer_allocator;
  v8::V8::SetArrayBufferAllocator(&array_buffer_allocator);
  v8::Isolate* isolate = v8::Isolate::New();
  run_shell = (argc == 1);
  int result;
  {
    v8::Isolate::Scope isolate_scope(isolate);
    v8::HandleScope handle_scope(isolate);
    v8::Handle<v8::Context> context = CreateShellContext(isolate);
    if (context.IsEmpty()) {
      fprintf(stderr, "Error creating context\n");
      return 1;
    }
    v8::Context::Scope context_scope(context);
    result = RunMain(isolate, argc, argv);
    if (run_shell) RunShell(context);
  }
  v8::V8::Dispose();
  v8::V8::ShutdownPlatform();
  delete platform;
  return result;
}


// Extracts a C string from a V8 Utf8Value.
const char* ToCString(const v8::String::Utf8Value& value) {
  return *value ? *value : "<string conversion failed>";
}


// Creates a new execution environment containing the built-in
// functions.
v8::Handle<v8::Context> CreateShellContext(v8::Isolate* isolate) {
  // Create a template for the global object.
  v8::Handle<v8::ObjectTemplate> global = v8::ObjectTemplate::New(isolate);
  // Bind the global 'print' function to the C++ Print callback.
  global->Set(v8::String::NewFromUtf8(isolate, "print"),
			  v8::FunctionTemplate::New(isolate,Print));
  // Bind the global 'read' function to the C++ Read callback.
  global->Set(v8::String::NewFromUtf8(isolate, "read"),
			  v8::FunctionTemplate::New(isolate,Read));
  // Bind the global 'load' function to the C++ Load callback.
  global->Set(v8::String::NewFromUtf8(isolate, "load"),
			  v8::FunctionTemplate::New(isolate,Load));
  // Bind the 'quit' function
  global->Set(v8::String::NewFromUtf8(isolate, "quit"),
			  v8::FunctionTemplate::New(isolate,Quit));
  // Bind the 'version' function
  global->Set(v8::String::NewFromUtf8(isolate, "version"),
			  v8::FunctionTemplate::New(isolate,Version));

  // Bind the handlers for external arrays.
#ifdef S8_EXPERIMENTAL
  global->Set(v8::String::NewFromUtf8(isolate, "Int8Array"), v8::FunctionTemplate::New(isolate,Int8Array));
  global->Set(v8::String::NewFromUtf8(isolate, "Uint8Array"), v8::FunctionTemplate::New(isolate,Uint8Array));
  global->Set(v8::String::NewFromUtf8(isolate, "Int16Array"), v8::FunctionTemplate::New(isolate,Int16Array));
  global->Set(v8::String::NewFromUtf8(isolate, "Uint16Array"), v8::FunctionTemplate::New(isolate,Uint16Array));
  global->Set(v8::String::NewFromUtf8(isolate, "Int32Array"), v8::FunctionTemplate::New(isolate,Int32Array));
  global->Set(v8::String::NewFromUtf8(isolate, "Uint32Array"), v8::FunctionTemplate::New(isolate,Uint32Array));
  global->Set(v8::String::NewFromUtf8(isolate, "Float32Array"), v8::FunctionTemplate::New(isolate,Float32Array));
  global->Set(v8::String::NewFromUtf8(isolate, "Float64Array"), v8::FunctionTemplate::New(isolate,Float64Array));
  global->Set(v8::String::NewFromUtf8(isolate, "PixelArray"), v8::FunctionTemplate::New(isolate,PixelArray));
#endif // S8_EXPERIMENTAL

  global->Set(v8::String::NewFromUtf8(isolate, "fileContents"),
			  v8::FunctionTemplate::New(isolate,FileContents));
  global->Set(v8::String::NewFromUtf8(isolate, "fileWrite"),
			  v8::FunctionTemplate::New(isolate,FileWrite));
  global->Set(v8::String::NewFromUtf8(isolate, "debug"),
			  v8::FunctionTemplate::New(isolate,DebugBreak));
  global->Set(v8::String::NewFromUtf8(isolate, "isFileOlderThan"),
			  v8::FunctionTemplate::New(isolate,IsFileOlderThan));

  return v8::Context::New(isolate, NULL, global);
}


// The callback that is invoked by v8 whenever the JavaScript 'print'
// function is called.  Prints its arguments on stdout separated by
// spaces and ending with a newline.
void Print(const v8::FunctionCallbackInfo<v8::Value>& args) {
  bool first = true;
  for (int i = 0; i < args.Length(); i++) {
    v8::HandleScope handle_scope(args.GetIsolate());
    if (first) {
      first = false;
    } else {
      printf(" ");
    }
    v8::String::Utf8Value str(args[i]);
    const char* cstr = ToCString(str);
    printf("%s", cstr);
  }
  printf("\n");
  fflush(stdout);
  //args.GetReturnValue().Set(NULL);
}


// The callback that is invoked by v8 whenever the JavaScript 'read'
// function is called.  This function loads the content of the file named in
// the argument into a JavaScript string.
void Read(const v8::FunctionCallbackInfo<v8::Value>& args) {
	if (args.Length() == 0) { // ReadLine from std input
		static const int kBufferSize = 8*1024;
		char buffer[kBufferSize];
		fprintf(stderr, "?");
		char* str = fgets(buffer, kBufferSize, stdin);
		if (str == NULL) return;
		args.GetReturnValue().Set(v8::String::NewFromUtf8(args.GetIsolate(), str));
		return;
	}
  if (args.Length() != 1) {
    args.GetIsolate()->ThrowException(v8::String::NewFromUtf8(args.GetIsolate(),"Bad parameters"));
	return;
  }
  v8::String::Utf8Value file(args[0]);
  if (*file == NULL) {
    args.GetIsolate()->ThrowException(v8::String::NewFromUtf8(args.GetIsolate(),"Error loading file"));
	return;
  }
  v8::Handle<v8::String> source = ReadFile(args.GetIsolate(),*file);
  if (source.IsEmpty()) {
    args.GetIsolate()->ThrowException(v8::String::NewFromUtf8(args.GetIsolate(),"Empty/missing file"));
	return;
  }
  args.GetReturnValue().Set(source);
}


// The callback that is invoked by v8 whenever the JavaScript 'load'
// function is called.  Loads, compiles and executes its argument
// JavaScript file.
void Load(const v8::FunctionCallbackInfo<v8::Value>& args) {
  for (int i = 0; i < args.Length(); i++) {
    v8::HandleScope handle_scope(args.GetIsolate());
    v8::String::Utf8Value file(args[i]);
    if (*file == NULL) {
      args.GetIsolate()->ThrowException(v8::String::NewFromUtf8(args.GetIsolate(),"Error loading file"));
	  return;
    }
    v8::Handle<v8::String> source = ReadFile(args.GetIsolate(),*file);
    if (source.IsEmpty()) {
      args.GetIsolate()->ThrowException(v8::String::NewFromUtf8(args.GetIsolate(),"Error loading file"));
	  return;
    }
    if (!ExecuteString(args.GetIsolate(),
                       source,
                       v8::String::NewFromUtf8(args.GetIsolate(),*file),
                       false,
                       true)) {
      args.GetIsolate()->ThrowException(v8::String::NewFromUtf8(args.GetIsolate(),"Error executing file"));
	  return;
    }
  }
  //args.GetReturnValue().Set(NULL);
}

// The callback that is invoked by v8 whenever the JavaScript 'quit'
// function is called.  Quits.
void Quit(const v8::FunctionCallbackInfo<v8::Value>& args) {
  // If not arguments are given args[0] will yield undefined which
  // converts to the integer value 0.
  int exit_code = args[0]->Int32Value();
  fflush(stdout);
  fflush(stderr);
  exit(exit_code);
  //args.GetReturnValue().Set(NULL);
}


void Version(const v8::FunctionCallbackInfo<v8::Value>& args) {
  args.GetReturnValue().Set(v8::String::NewFromUtf8(args.GetIsolate(),v8::V8::GetVersion()));
}


// Reads a file into a v8 string.
v8::Handle<v8::String> ReadFile(v8::Isolate* isolate, const char* name) {
  FILE* file = fopen(name, "rb");
  if (file == NULL) return v8::Handle<v8::String>();

  fseek(file, 0, SEEK_END);
  int size = ftell(file);
  rewind(file);

  char* chars = new char[size + 1];
  chars[size] = '\0';
  for (int i = 0; i < size;) {
    int read = static_cast<int>(fread(&chars[i], 1, size - i, file));
    i += read;
  }
  fclose(file);
  v8::Handle<v8::String> result = v8::String::NewFromUtf8(isolate, chars, v8::String::kNormalString, size);
  delete[] chars;
  return result;
}


// The callback that is invoked to get contents of a file
void FileContents(const v8::FunctionCallbackInfo<v8::Value>& args) {
    if (args.Length() != 1) {
		args.GetIsolate()->ThrowException(v8::Exception::Error(v8::String::NewFromUtf8(args.GetIsolate(),"Bad parameters")));
		return;
    }
	v8::String::Utf8Value file(args[0]);
    if (*file == NULL) {
      args.GetIsolate()->ThrowException(v8::Exception::Error(v8::String::NewFromUtf8(args.GetIsolate(),"Missing fileName argument")));
	  return;
    }
    v8::Handle<v8::String> source = ReadFile(args.GetIsolate(),*file);
	args.GetReturnValue().Set(source);
}

// The callback that is invoked to output string to a file
void FileWrite(const v8::FunctionCallbackInfo<v8::Value>& args) {
    if (args.Length() != 3) {
		args.GetIsolate()->ThrowException(v8::Exception::Error(v8::String::NewFromUtf8(args.GetIsolate(),"Bad parameters")));
		return;
    }
	v8::String::Utf8Value fileName(args[0]);
	v8::String::Utf8Value aString(args[1]);
	v8::String::Utf8Value mode(args[2]);
	int written = 0;
	FILE* f = fopen(*fileName, *mode);
	if (f == NULL) {
		args.GetIsolate()->ThrowException(v8::Exception::Error(v8::String::NewFromUtf8(args.GetIsolate(),"Fail to open file")));
		return;
	}
	int size = aString.length();
	char* str = *aString;
	int write = size;
	while ((write > 0) && (written < size)) {
		write = static_cast<int>(fwrite(str, 1, size - written, f));
		written += write;
		str += write;
	}
	fclose(f);
	args.GetReturnValue().Set(v8::Integer::New(args.GetIsolate(),written));
}


// The callback that is invoked by v8 whenever the JavaScript 'isFileOlderThan'
// function is called.  Return true if file at first argument exists and is older than reference file (second argument).
void IsFileOlderThan(const v8::FunctionCallbackInfo<v8::Value>& args) {
	if (args.Length() != 2) {
	    args.GetIsolate()->ThrowException(v8::String::NewFromUtf8(args.GetIsolate(),"Bad parameters"));
		return;
	}
	args.GetReturnValue().Set(false);
	v8::String::Utf8Value fileName(args[0]);
	if (*fileName == NULL) return;
	v8::String::Utf8Value referenceName(args[1]);
	if (*referenceName == NULL) return;

	struct stat fattrib;
	if (stat(*fileName, &fattrib) != 0) return;
	struct stat rattrib;
	if (stat(*referenceName, &rattrib) != 0) return;

	args.GetReturnValue().Set(fattrib.st_mtime < rattrib.st_mtime);
}

// The callback that is invoked to enter debugger
void DebugBreak(const v8::FunctionCallbackInfo<v8::Value>& args) {
#ifdef _MSC_VER
  __debugbreak();
#else
  ::DebugBreak(args);
#endif
	// args.GetReturnValue().Set(v8::Undefined());
}

// Returns a v8 string of specified size.
void BasicStringNew(const v8::FunctionCallbackInfo<v8::Value>& args) {
	if (args.Length() != 1) {
		args.GetIsolate()->ThrowException(v8::Exception::Error(v8::String::NewFromUtf8(args.GetIsolate(),"Bad parameters")));
		return;
    }
	int size = args[0]->Int32Value();
	char* chars = new char[size + 1];
	//for (int i = 0; i <= size; i++) { chars[i]='\0'; }
	memset(chars, 0, size + 1);
	v8::Handle<v8::String> result = v8::String::NewFromUtf8(args.GetIsolate(), chars, v8::String::kNormalString, size);
	delete[] chars;
	args.GetReturnValue().Set(result);
}


// Process remaining command line arguments and execute files
int RunMain(v8::Isolate* isolate, int argc, char* argv[]) {
  for (int i = 1; i < argc; i++) {
    const char* str = argv[i];
    if (strcmp(str, "--shell") == 0) {
      run_shell = true;
    } else if (strcmp(str, "-f") == 0) {
      // Ignore any -f flags for compatibility with the other stand-
      // alone JavaScript engines.
      continue;
    } else if (strncmp(str, "--", 2) == 0) {
      fprintf(stderr,
              "Warning: unknown flag %s.\nTry --help for options\n", str);
    } else if (strcmp(str, "-e") == 0 && i + 1 < argc) {
      // Execute argument given to -e option directly.
      v8::Handle<v8::String> file_name =
		  v8::String::NewFromUtf8(isolate,"unnamed");
      v8::Handle<v8::String> source =
		  v8::String::NewFromUtf8(isolate,argv[++i]);
      if (!ExecuteString(isolate, source, file_name, false, true)) return 1;
    } else {
      // Use all other arguments as names of files to load and run.
      v8::Handle<v8::String> file_name = v8::String::NewFromUtf8(isolate,str);
      v8::Handle<v8::String> source = ReadFile(isolate,str);
      if (source.IsEmpty()) {
        fprintf(stderr, "Error reading '%s'\n", str);
        continue;
      }
      if (!ExecuteString(isolate, source, file_name, false, true)) return 1;
    }
  }
  return 0;
}


// The read-eval-execute loop of the shell.
void RunShell(v8::Handle<v8::Context> context) {
  fprintf(stderr, "s8vm (V8 %s)\n", v8::V8::GetVersion());
  static const int kBufferSize = 256;
  // Enter the execution environment before evaluating any code.
  v8::Context::Scope context_scope(context);
  v8::Local<v8::String> name(v8::String::NewFromUtf8(context->GetIsolate(), "(shell)"));
  while (true) {
    char buffer[kBufferSize];
    fprintf(stderr, "> ");
    char* str = fgets(buffer, kBufferSize, stdin);
    if (str == NULL) break;
    v8::HandleScope handle_scope(context->GetIsolate());
    ExecuteString(context->GetIsolate(),
                  v8::String::NewFromUtf8(context->GetIsolate(), str),
                  name,
                  true,
                  true);
  }
  fprintf(stderr, "\n");
}


// Executes a string within the current v8 context.
bool ExecuteString(v8::Isolate* isolate,
                   v8::Handle<v8::String> source,
                   v8::Handle<v8::Value> name,
                   bool print_result,
                   bool report_exceptions) {
  v8::HandleScope handle_scope(isolate);
  v8::TryCatch try_catch;
  v8::ScriptOrigin origin(name);
  v8::Handle<v8::Script> script = v8::Script::Compile(source, &origin);
  if (script.IsEmpty()) {
    // Print errors that happened during compilation.
    if (report_exceptions)
      ReportException(isolate, &try_catch);
    return false;
  } else {
    v8::Handle<v8::Value> result = script->Run();
    if (result.IsEmpty()) {
      assert(try_catch.HasCaught());
      // Print errors that happened during execution.
      if (report_exceptions)
        ReportException(isolate, &try_catch);
      return false;
    } else {
      assert(!try_catch.HasCaught());
      if (print_result && !result->IsUndefined()) {
        // If all went well and the result wasn't undefined then print
        // the returned value.
        v8::String::Utf8Value str(result);
        const char* cstr = ToCString(str);
        printf("%s\n", cstr);
      }
      return true;
    }
  }
}


void ReportException(v8::Isolate* isolate, v8::TryCatch* try_catch) {
  v8::HandleScope handle_scope(isolate);
  v8::String::Utf8Value exception(try_catch->Exception());
  const char* exception_string = ToCString(exception);
  v8::Handle<v8::Message> message = try_catch->Message();
  if (message.IsEmpty()) {
    // V8 didn't provide any extra information about this error; just
    // print the exception.
    fprintf(stderr, "%s\n", exception_string);
  } else {
    // Print (filename):(line number): (message).
	  v8::String::Utf8Value filename(message->GetScriptOrigin().ResourceName());
    const char* filename_string = ToCString(filename);
    int linenum = message->GetLineNumber();
    fprintf(stderr, "%s:%i: %s\n", filename_string, linenum, exception_string);
    // Print line of source code.
    v8::String::Utf8Value sourceline(message->GetSourceLine());
    const char* sourceline_string = ToCString(sourceline);
    fprintf(stderr, "%s\n", sourceline_string);
    // Print wavy underline (GetUnderline is deprecated).
    int start = message->GetStartColumn();
    for (int i = 0; i < start; i++) {
      fprintf(stderr, " ");
    }
    int end = message->GetEndColumn();
    for (int i = start; i < end; i++) {
      fprintf(stderr, "^");
    }
    fprintf(stderr, "\n");
    v8::String::Utf8Value stack_trace(try_catch->StackTrace());
    if (stack_trace.length() > 0) {
      const char* stack_trace_string = ToCString(stack_trace);
      fprintf(stderr, "%s\n", stack_trace_string);
    }
  }
}


#ifdef S8_EXPERIMENTAL
void ExternalArrayWeakCallback(v8::Persistent<v8::Value> object, void* data) {
  free(data);
  object.Dispose();
}

void CreateExternalArray(const v8::FunctionCallbackInfo<v8::Value>& args,
                                          v8::ExternalArrayType type,
                                          int element_size) {
  if (args.Length() != 1) {
    v8::ThrowException(v8::Exception::Error(v8::String::New("Array constructor needs one parameter.")));
	return;
  }
  int length = args[0]->Int32Value();
  void* data = malloc(length * element_size);
  memset(data, 0, length * element_size);
  v8::Handle<v8::Object> array = v8::Object::New();
  v8::Persistent<v8::Object> persistent_array = v8::Persistent<v8::Object>::New(array);
  persistent_array.MakeWeak(data, ExternalArrayWeakCallback);
  persistent_array.MarkIndependent();
  array->SetIndexedPropertiesToExternalArrayData(data, type, length);
  array->Set(v8::String::New("length"), v8::Int32::New(length), v8::ReadOnly);
  array->Set(v8::String::New("BYTES_PER_ELEMENT"), v8::Int32::New(element_size));
  args.GetReturnValue().Set(array);
}


void Int8Array(const v8::FunctionCallbackInfo<v8::Value>& args) {
	CreateExternalArray(args, v8::kExternalByteArray, sizeof(int8_t));
}


void Uint8Array(const v8::FunctionCallbackInfo<v8::Value>& args) {
	CreateExternalArray(args, v8::kExternalUnsignedByteArray, sizeof(uint8_t));
}


void Int16Array(const v8::FunctionCallbackInfo<v8::Value>& args) {
	CreateExternalArray(args, v8::kExternalShortArray, sizeof(int16_t));
}


void Uint16Array(const v8::FunctionCallbackInfo<v8::Value>& args) {
	CreateExternalArray(args, v8::kExternalUnsignedShortArray, sizeof(uint16_t));
}

void Int32Array(const v8::FunctionCallbackInfo<v8::Value>& args) {
	CreateExternalArray(args, v8::kExternalIntArray, sizeof(int32_t));
}


void Uint32Array(const v8::FunctionCallbackInfo<v8::Value>& args) {
	CreateExternalArray(args, v8::kExternalUnsignedIntArray, sizeof(uint32_t));
}


void Float32Array(const v8::FunctionCallbackInfo<v8::Value>& args) {
	CreateExternalArray(args, v8::kExternalFloatArray, sizeof(float));  // NOLINT
}


void Float64Array(const v8::FunctionCallbackInfo<v8::Value>& args) {
	CreateExternalArray(args, v8::kExternalDoubleArray, sizeof(double));  // NOLINT
}


void PixelArray(const v8::FunctionCallbackInfo<v8::Value>& args) {
	CreateExternalArray(args, v8::kExternalPixelArray, sizeof(uint8_t));
}

#endif // S8_EXPERIMENTAL
